/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4;

import org.xj4.spi.XJ4ExtensionProvider;
import org.xj4.spi.MethodHandlerFactory;
import org.xj4.spi.MethodWrapperSource;
import org.xj4.spi.ClassWrapperSource;
import org.xj4.spi.TestClassValidator;
import org.xj4.util.ServiceLoader;

import java.util.List;
import java.util.ArrayList;
import java.io.IOException;

import org.apache.commons.collections15.Transformer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * An implementation of XJ4ExtensionProvider that looks up other providers in the ServiceLoader, aggregates them, and
 * provides their results as it's own.
 */
class ServiceExtensionProvider extends XJ4ExtensionProvider {

  @SuppressWarnings({"UnusedDeclaration"})
  private static Log logger = LogFactory.getLog(ServiceExtensionProvider.class);

  private ServiceLoader<XJ4ExtensionProvider> providers;

  public ServiceExtensionProvider() throws IOException {
    this.providers = new ServiceLoader(XJ4ExtensionProvider.class);
  }

  public List<MethodHandlerFactory> getMethodHandlerFactories() {
    return aggregate(new ProviderCollector<MethodHandlerFactory>() {
      public List<MethodHandlerFactory> transform(XJ4ExtensionProvider xj4ExtensionProvider) {
        return xj4ExtensionProvider.getMethodHandlerFactories();
      }
    });
  }

  public List<MethodWrapperSource> getMethodWrapperSources() {
    return aggregate(new ProviderCollector<MethodWrapperSource>() {
      public List<MethodWrapperSource> transform(XJ4ExtensionProvider xj4ExtensionProvider) {
        return xj4ExtensionProvider.getMethodWrapperSources();
      }
    });
  }

  public List<ClassWrapperSource> getClassWrapperSources() {
    return aggregate(new ProviderCollector<ClassWrapperSource>() {
      public List<ClassWrapperSource> transform(XJ4ExtensionProvider xj4ExtensionProvider) {
        return xj4ExtensionProvider.getClassWrapperSources();
      }
    });
  }

  public List<TestClassValidator> getTestClassValidators() {
    return aggregate(new ProviderCollector<TestClassValidator>() {
      public List<TestClassValidator> transform(XJ4ExtensionProvider xj4ExtensionProvider) {
        return xj4ExtensionProvider.getTestClassValidators();
      }
    });
  }

  private <T> List<T> aggregate(Transformer<XJ4ExtensionProvider, List<T>> retriever) {
    List<T> aggregation = new ArrayList<T>();
    for(XJ4ExtensionProvider provider: providers) {
      aggregation.addAll(retriever.transform(provider));
    }
    return aggregation;
  }

  private interface ProviderCollector<T> extends Transformer<XJ4ExtensionProvider, List<T>> {

  }
}
