/**
 * Copyright 2008 Peach Jean Solutions
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.util;

import java.util.List;
import java.util.Comparator;

/**
 * Created by IntelliJ IDEA.
 * User: jbunting
 * Date: Nov 13, 2008
 * Time: 10:33:21 AM
 * To change this template use File | Settings | File Templates.
 */
public interface Orderable {

  /**
   * This returns a list of the classes of objects that this one must come BEFORE.
   * @return
   */
  public List<Class<?>> before();

  /**
   * This returns a list of the classes of objects that this one must come AFTER.
   * @return
   */
  public List<Class<?>> after();

  public boolean beforeAll();

  public boolean afterAll();

  public static class OrderableComparator implements Comparator<Object> {
    public int compare(Object o1, Object o2) {

      int order = sortOrders(determinesOrder(o1, o2), determinesOrder(o2, o1), false);

      if(order == 0) {
        order = sortOrders(determinesOrderAll(o1, o2), determinesOrderAll(o2, o1), true);
      }

      return order;
    }

    private int sortOrders(int o1Order, int o2Order, boolean ignoreConflict) {
      if(o1Order == 0 && o2Order == 0) {
        return 0;
      }

      if(o1Order == o2Order) {
        if(ignoreConflict) {
          return 0;
        } else {
          throw new IllegalStateException("Conflicting Ordering.  I can't handle this.");
        }
      }

      if(o1Order != 0) {
        return o1Order;
      } else {
        return o2Order;
      }
    }

    private int determinesOrderAll(Object o1, Object o2) {
      if(o1 instanceof Orderable) {
        boolean before = ((Orderable)o1).beforeAll();
        boolean after = ((Orderable)o1).afterAll();
        if(before && after) {
          throw new IllegalStateException(String.format("%s wants to come before AND after everything.  Ain't gonna happen.", o1));
        } else if(before) {
          return -1;
        } else if(after) {
          return 1;
        }
      }
      return 0;
    }

    private int determinesOrder(Object o1, Object o2) {
      if(o1 instanceof Orderable) {
        Orderable orderable = (Orderable) o1;
        boolean isBefore = contains(orderable.before(), o2.getClass());
        boolean isAfter = contains(orderable.after(), o2.getClass());
        if(isBefore && isAfter) {
          throw new IllegalStateException(String.format("%s specifies that %s must become before AND after it.  This is not possible.", o1, o2));
        }
        if(isBefore) {
          return -1;
        } else if(isAfter) {
          return 1;
        }
      }
      return 0;
    }

    private boolean contains(List<Class<?>> classes, Class<?> test) {
      for(Class<?> clazz: classes) {
        if(clazz.isAssignableFrom(test)) {
          return true;
        }
      }
      return false;
    }
  }
}
